-------------------------------------------------------------------------------
--=> Frames.lua
-- weizPVP: Frames
-- Creates and updates the addon frames and widgets
-------------------------------------------------------------------------------
--> luacheck
-- luacheck: no max line length
-- luacheck: ignore  _.* for k self
--> weizPVP: Libs
-- luacheck: globals LibStub
--> weizPVP: Globals
-- luacheck: globals weizPVP  weizPVP weizPVP_DB weizPVP:DebugPrint weizPVP.Settings ViragDevTool_AddData
-- luacheck: globals weizPVP.CoreFrame weizPVP.HeaderFrame weizPVP.ScrollFrame weizPVP.ListFrame weizPVPScrollOffset
--> WoW API Globals
-- luacheck: globals ChatFrame7 CreateFrame UIParent  InCombatLockdown
-- luacheck: globals truncate getn
-------------------------------------------------------------------------------
--=> Import
-------------------------------------------------------------------------------
local weizPVP = weizPVP
-------------------------------------------------------------------------------
--=> LIBS
-------------------------------------------------------------------------------
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")

-------------------------------------------------------------------------------
--=> LOCALS
-------------------------------------------------------------------------------
local BGCOLOR = {
    ["R"] = weizPVP.Settings.BACKGROUNDCOLOR.R,
    ["G"] = weizPVP.Settings.BACKGROUNDCOLOR.G,
    ["B"] = weizPVP.Settings.BACKGROUNDCOLOR.B,
    ["A"] = weizPVP.Settings.BACKGROUNDCOLOR.A
}

-------------------------------------------------------------------------------
--=> FUNCTIONS
-------------------------------------------------------------------------------

function weizPVP:RepositionCoreFrame()
    if not InCombatLockdown() then
        local top = weizPVP:Round(weizPVP.CoreFrame:GetTop())
        local left = weizPVP.CoreFrame:GetLeft()
        local height = weizPVP:Round(weizPVP.CoreFrame:GetHeight())
        local width = weizPVP:Round(weizPVP.CoreFrame:GetWidth())
        weizPVP.CoreFrame:ClearAllPoints()
        weizPVP.CoreFrame:SetPoint("TOPLEFT", "UIParent", "BOTTOMLEFT", left, top)
        weizPVP.CoreFrame:SetHeight(height)
        weizPVP.CoreFrame:SetWidth(width)
    end
end

function weizPVP:ReadjustBars(self, value)
    if not InCombatLockdown() then
        local offsetMax = weizPVP:Round(weizPVP.ListFrame:GetHeight() - weizPVP.ScrollFrame:GetHeight(), 0)
        local offset =
            weizPVP:Round((weizPVP.ScrollFrame:GetVerticalScroll()) + (-1 * value * weizPVP.Settings.LIST.ROWHEIGHT), 0)
        if offset < 0 then
            weizPVP.ScrollFrame:SetVerticalScroll(0)
        elseif offset > offsetMax then
            -- weizPVP:FitFrameToBars()
            weizPVP.ScrollFrame:SetVerticalScroll(offsetMax)
        else
            weizPVP.ScrollFrame:SetVerticalScroll(offset)
        end
    end
end

-------------------------------------------------------------------------------
--|> CONFIGURE FRAMES
-------------------------------------------------------------------------------

--=> weizPVP.CoreFrame: SETTINGS
-->
-----------------------------------------------------------------------
weizPVP.CoreFrame = CreateFrame("Frame", "weizPVP.CoreFrame", UIParent)
weizPVP.CoreFrame:SetClampedToScreen(false)
weizPVP.CoreFrame:SetFrameStrata("MEDIUM")
weizPVP.CoreFrame:SetFrameLevel(1)
weizPVP.CoreFrame:SetPoint("CENTER", UIParent)
weizPVP.CoreFrame:SetSize(weizPVP.Settings.MAIN.WIDTH, weizPVP.Settings.MAIN.HEIGHT - weizPVP.Settings.BARS.SPACING)
weizPVP.CoreFrame:SetMovable(true)
weizPVP.CoreFrame:EnableMouse(true)
weizPVP.CoreFrame:SetBackdrop(
    {
        bgFile = SM:Fetch("background", "weizPVP-Background"),
        tile = false,
        edgeFile = SM:Fetch("border", "weizPVP-Border"),
        edgeSize = 1
    }
)
weizPVP.CoreFrame:SetBackdropColor(BGCOLOR.R, BGCOLOR.G, BGCOLOR.B, 0.6)
weizPVP.CoreFrame:SetBackdropBorderColor(0, 0, 0, 1)
weizPVP.CoreFrame:SetResizable(true)
weizPVP.CoreFrame:SetMinResize(180, weizPVP.Settings.LIST.ROWHEIGHT * 1)
weizPVP.CoreFrame:SetMaxResize(
    500,
    (weizPVP.Settings.LIST.ROWHEIGHT * weizPVP.Settings.BARS.MAXNUMBER) - weizPVP.Settings.BARS.SPACING
)
weizPVP.CoreFrame:Show()

--=> weizPVP.ScrollFrame: SETTINGS
--> The frame that is a child of weizPVP.CoreFrame that will be scolling with content
-----------------------------------------------------------------------
weizPVP.ScrollFrame = CreateFrame("ScrollFrame", "weizPVP.ScrollFrame", weizPVP.CoreFrame, "weizPVPSecureScrollFrame")
weizPVP.ScrollFrame:EnableMouseWheel(true)
weizPVP.ScrollFrame:SetClipsChildren(true)
weizPVP.ScrollFrame:SetPoint("TOPLEFT", weizPVP.CoreFrame, "TOPLEFT", 4, 0)
weizPVP.ScrollFrame:SetPoint("BOTTOMRIGHT", weizPVP.CoreFrame, "BOTTOMRIGHT", -4, 0)

--> MOUSEWHEEL
weizPVP.ScrollFrame:SetScript(
    "OnMouseWheel",
    function(self, value)
        weizPVP:ReadjustBars(self, value)
    end
)

--=> weizPVP.ListFrame: SETTINGS
--> The frame that is a child of weizPVP.CoreFrame that will be scolling with content
-----------------------------------------------------------------------
weizPVP.ListFrame = CreateFrame("Frame", "weizPVP.ListFrame", weizPVP.ScrollFrame)
weizPVP.ListFrame:SetPoint("CENTER", weizPVP.ScrollFrame, "CENTER")
weizPVP.ListFrame:SetHeight(weizPVP.Settings.LIST.HEIGHT)
weizPVP.ListFrame:SetWidth(weizPVP.ScrollFrame:GetWidth() - weizPVP.Settings.MAIN.BORDER_SIZE)
weizPVP.ListFrame.content = weizPVP.ListFrame
weizPVP.ScrollFrame:SetScrollChild(weizPVP.ListFrame)

--=> weizPVP.HeaderFrame: SETTINGS
-----------------------------------------------------------------------
weizPVP.HeaderFrame = CreateFrame("Frame", "weizPVP.HeaderFrame", UIParent)
weizPVP.HeaderFrame:SetClampedToScreen(false)
weizPVP.HeaderFrame:SetPoint("BOTTOMLEFT", weizPVP.CoreFrame, "TOPLEFT")
weizPVP.HeaderFrame:SetPoint("BOTTOMRIGHT", weizPVP.CoreFrame, "TOPRIGHT")
weizPVP.HeaderFrame:SetHeight(weizPVP.Settings.HEADER.HEIGHT)
weizPVP.HeaderFrame:EnableMouse(true)
weizPVP.HeaderFrame:SetBackdrop(
    {
        bgFile = SM:Fetch("background", "weizPVP-Background"),
        tile = false,
        edgeFile = SM:Fetch("border", "weizPVP-Border"),
        edgeSize = 1,
        insets = {left = 0, right = 0, top = 0, bottom = 0}
    }
)
weizPVP.HeaderFrame:SetBackdropColor(0.04, 0.04, 0.04, 0.8)
weizPVP.HeaderFrame:SetBackdropBorderColor(0, 0, 0, 1)

weizPVP.HeaderFrame:SetScript(
    "OnMouseDown",
    function(self, event)
        if (((not weizPVP.CoreFrame.isLocked) or (weizPVP.CoreFrame.isLocked == 0)) and (event == "LeftButton")) then
            weizPVP.CoreFrame:StartMoving()
            weizPVP.CoreFrame.isMoving = true
        end
    end
)

weizPVP.HeaderFrame:SetScript(
    "OnMouseUp",
    function(self)
        if (weizPVP.CoreFrame.isMoving) then
            weizPVP.CoreFrame:StopMovingOrSizing()
            weizPVP.CoreFrame.isMoving = false
            weizPVP:RepositionCoreFrame()
        --!! SaveMainWindowPosition()
        end
    end
)

--=> weizPVP.HeaderFrame.Text: SETTINGS
-----------------------------------------------------------------------
weizPVP.HeaderFrame.Text = weizPVP.HeaderFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
weizPVP.HeaderFrame.Text:SetPoint("LEFT", weizPVP.HeaderFrame, "LEFT", 8, 0)
weizPVP.HeaderFrame.Text:SetFont(
    SM:Fetch("font", weizPVP.Settings.HEADER.TEXT.FONT),
    weizPVP.Settings.HEADER.TEXT.FONTSIZE,
    weizPVP.Settings.HEADER.TEXT.FONTOUTLINE
)
weizPVP.HeaderFrame.Text:SetJustifyH("LEFT")
weizPVP.HeaderFrame.Text:SetJustifyV("MIDDLE")
weizPVP.HeaderFrame.Text:SetTextColor(1, 1, 1, 1)
weizPVP.HeaderFrame.Text:SetText("NEARBY |cff777777:|r |cFF26FF540|r")

--=> weizPVP. Create Header Icons
-----------------------------------------------------------------------
weizPVP:CreateHeaderIcons()

--=> weizPVP.FooterFrame: SETTINGS
-----------------------------------------------------------------------
weizPVP.FooterFrame = CreateFrame("Frame", "weizPVP.FooterFrame", weizPVP.CoreFrame)
weizPVP.FooterFrame:SetClampedToScreen(false)
weizPVP.FooterFrame:SetPoint("TOPLEFT", weizPVP.CoreFrame, "BOTTOMLEFT")
weizPVP.FooterFrame:SetPoint("TOPRIGHT", weizPVP.CoreFrame, "BOTTOMRIGHT")
weizPVP.FooterFrame:EnableMouse(true)
weizPVP.FooterFrame:SetHeight(weizPVP.Settings.FOOTER.HEIGHT)
weizPVP.FooterFrame:SetBackdrop(
    {
        bgFile = SM:Fetch("background", "weizPVP-Background"),
        tile = false,
        edgeFile = SM:Fetch("border", "weizPVP-Border"),
        edgeSize = 1,
        insets = {left = 0, right = 0, top = 0, bottom = 0}
    }
)
weizPVP.FooterFrame:SetBackdropColor(0.04, 0.04, 0.04, 0.8)
weizPVP.FooterFrame:SetBackdropBorderColor(0, 0, 0, 1)

--=> weizPVP.FooterFrame.Text: SETTINGS
-----------------------------------------------------------------------
weizPVP.FooterFrame.Text = weizPVP.FooterFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
weizPVP.FooterFrame.Text:SetPoint("TOPLEFT", weizPVP.FooterFrame, "TOPLEFT", 2, 0)
weizPVP.FooterFrame.Text:SetPoint("BOTTOMRIGHT", weizPVP.FooterFrame, "BOTTOMRIGHT")
weizPVP.FooterFrame.Text:SetFont(
    SM:Fetch("font", weizPVP.Settings.FOOTER.TEXT.FONT),
    weizPVP.Settings.FOOTER.TEXT.FONTSIZE,
    weizPVP.Settings.FOOTER.TEXT.FONTOUTLINE
)
weizPVP.FooterFrame.Text:SetJustifyH("LEFT")
weizPVP.FooterFrame.Text:SetJustifyV("MIDDLE")
weizPVP.FooterFrame.Text:SetText("")

weizPVP:CreateRezieCornre()

--|> CREATE BARS
-----------------------------------------------------------------------
weizPVP.ScrollFrame:SetVerticalScroll(0)
weizPVP:CreateBars()
weizPVP:RepositionCoreFrame()
